@@ -2,7 +2,7 @@
 use warnings;
 use strict;
 
-our ($ada_dir) = "/home/briot/Ada/GtkAda/src/";
+our ($ada_dir) = ".";
 #$c_dir  ="/home/briot/gtk/gtk+-2.9/gtk+-2.9.0/";
 our ($c_dir)   = "/home/briot/gtk/gtk+-2.8/gtk+-2.8.17";
 
@@ -51,6 +51,9 @@ our (%c_files_no_binding) =
    "gtktextiterprivate"     => 1, # Internal only
    "gtktexttagprivate"      => 1, # Internal only
    "gtktextmarkprivate"     => 1, # Internal only
+   "gtkfilechooserentry"    => 1, # Internal only
+   "gtkfilechooserdefault"  => 1, # Internal only
+   "gtkoldeditable"         => 1, # Deprecated
    "gtktree"        => 1,  # Broken
    "gtktreeitem"    => 1,  # Broken
    "gtkrbtree"      => 1,  # Internal only
@@ -75,11 +78,24 @@ our (%c_files_no_binding) =
    "gtkwindow-decorate" => 1,  # Never used anywhere, no ref on google... for gtkwindow.c
    "gtkaccessible"  => 1, # Small interface to ATK, which we do not bind anyway
    "gtkdndcursors"  => 1, # Internal to the DND implementation
+   "gtkiconcache"   => 1, # Internal
    "xembed"         => 1,
    "gtkplugprivate" => 1,
 
-   # We might provide a binding for those one day, but they are very secondary
-   "gtkimmodule"    => 1,
+   # We might provide a binding for those one day, but they are very secondary. They are
+   # also not documented in the gtk+ manual itself
+   "gtkimmodule"         => 1,
+   "gtkfilechooserembed" => 1,
+   "gtkfilechooserutils" => 1,
+   "gtktypebuiltins"     => 1, 
+   "gtkimcontext"        => 1,
+   "gtkimcontextsimple"  => 1,
+   "gtkimmulticontext"   => 1,
+   "gtkhsv"              => 1,
+   "gtkfilesystem"       => 1, # Hard to bind, requires support for external
+   "gtkfilesystemmodel"  => 1, # modules in gtk+ apparently, see
+   "gtkfilesystemunix"   => 1, # _gtk_file_system_create, and
+   "gtkfilesystemwin32"  => 1  # set_file_system_backend
    );
 
 ## Return the base name (no extension) for a C file
@@ -105,6 +121,7 @@ sub ada_unit_from_c_file() {
   my ($adafile) = &basename ($cfile);
   my (@words);
   $adafile =~ s/^gtk(.+)/gtk-$1/;
+  $adafile =~ s/^gdk(.+)/gdk-$1/;
 
   ## Order matters in this array
   @words = ("about", "accel", "action", "aspect", "button", "cell", "check",
@@ -129,11 +146,13 @@ sub ada_unit_from_c_file() {
   $adafile =~ s/^gmain$/glib-main/;
   $adafile =~ s/-tipsquery/-tips_query/;
   $adafile =~ s/-spinbutton/-spin_button/;
-  $adafile =~ s/-oldeditable/-old_editable/;
+  #$adafile =~ s/-oldeditable/-old_editable/;   ## Ignore this one, always
   $adafile =~ s/-range/-grange/;
   $adafile =~ s/-imagemenuitem/-image_menu_item/;
   $adafile =~ s/-gamma/-gamma_curve/;
   $adafile =~ s/-plotcanvas(.+)/-extra-plot_canvas-$1/;
+  $adafile =~ s/uimanager/ui_manager/;
+  $adafile =~ s/__/_/g;
   return $adafile;
 }
 
@@ -215,9 +234,12 @@ our $param_spec_re = '\s*g_param_spec_(\w+)\s*\("([^"]+)"';
 our $property_re =
   'g_object_class_install_property\s*\(\s*\w+\s*,\s*\w+\s*,';
 
+our $interface_property_re = 
+  'g_object_interface_install_property\s*\(\s*\w+\s*,';
+
 # no parenthesis
 our $style_property_re =
-  'gtk_widget_class_install_style_property\s*\(\s*\w+\s*,';
+  'gtk_widget_(?:class|interface)_install_style_property\s*\(\s*\w+\s*,';
 
 # no parenthesis
 our $child_property_re =
@@ -242,7 +264,7 @@ sub properties_in_c_file() {
    my ($content) = &get_c_file_content ($fullname);
 
    while ($content =~
-            /(?:($property_re|$style_property_re|$child_property_re)$param_spec_re$property_descr_re)|(?:$c_signal_re)|(?:$c_doc_re)/ogs)
+            /(?:($property_re|$interface_property_re|$style_property_re|$child_property_re)$param_spec_re$property_descr_re)|(?:$c_signal_re)|(?:$c_doc_re)/ogs)
    {
       my ($propcategory, $proptype, $propname, $propdescr,
           $signame,
@@ -400,7 +422,7 @@ sub get_c_file_content () {
 
 ## Find out all C functions defined in a C file.
 ## Return a hash table indexed on the functions
-our $c_function_re = '\b(\w+(\s*\*+|\s))\s*(\w+)\s*\(([^)]*\))(\s*G_GNUC_CONST)?;';
+our $c_function_re = '\b(\w+(\s*\*+|\s))\s*(\w+)\s*\(([^)]*\))(\s*G_GNUC_CONST|\s*G_GNUC_NULL_TERMINATED|\s*G_GNUC_PRINTF\s+\(\d+,\s*\d+\))?;';
 our $c_deprecated_re = '(if |ifndef|endif).*GTK_DISABLE_DEPRECATED';
 our $c_broken_re     = '\#(ifdef.*GTK_ENABLE_BROKEN|endif)';
 sub functions_from_c_file() {
@@ -457,8 +479,13 @@ sub is_object() {
    return (($c_type =~ /^Gtk(?:.+)\*/
                && $c_type ne "GtkClipboard*"
                && $c_type ne "GtkTreeSortable*"
+               && $c_type ne "GtkFileChooser*"
+               && $c_type ne "GtkTreeDragSource*"
+               && $c_type ne "GtkTreeDragDest*"
                && $c_type ne "GtkCellLayout*")
            || $c_type eq "PangoLayout"
+           || $c_type eq "GdkDisplay*"
+           || $c_type eq "GdkScreen*"
            || $c_type eq "GObject*");
 }
 
@@ -470,7 +497,6 @@ sub c_to_ada() {
    $c_type =~ s/([^_])([A-Z])/$1_$2/g;  ## Split on upper cases
 
    return "Boolean"            if ($c_type eq "gboolean");
-   return "Gdk_$1"             if ($c_type =~ /Gdk_(.+)\*/);
    return "Gfloat"             if ($c_type eq "gfloat");
    return "String"             if ($c_type =~ /g?char\*/);
    return "Gtk_Tree_Iter"      if ($c_type eq "Gtk_Tree_Iter*");
@@ -484,7 +510,11 @@ sub c_to_ada() {
    return "GObject"            if ($param_index == -1 && $c_type eq "G_Object*");
    return "Gtk_Clipboard"      if ($c_type eq "Gtk_Clipboard*");
    return "Gtk_Cell_Layout"    if ($c_type eq "Gtk_Cell_Layout*");
+   return "Gtk_Tree_Drag_Dest" if ($c_type eq "Gtk_Tree_Drag_Dest*");
+   return "Gtk_Tree_Drag_Source" if ($c_type eq "Gtk_Tree_Drag_Source*");
+   return "Gtk_File_Chooser"   if ($c_type eq "Gtk_File_Chooser*");
    return "Gtk_Tree_Sortable"  if ($c_type eq "Gtk_Tree_Sortable*");
+   return "Selection_Data"     if ($c_type eq "Gtk_Selection_Data*");
 
    if ($is_object) {
       if ($param_index == -1) {
@@ -495,6 +525,8 @@ sub c_to_ada() {
          return "access " . &c_widget_to_ada ($c_type) . "_Record'Class";
       }
    }
+
+   return "Gdk_$1"             if ($c_type =~ /Gdk_(.+)\*/);
    return &capitalize ($c_type);
 }
 
@@ -511,9 +543,10 @@ sub c_to_low_ada() {
    return "Gfloat"         if ($c_type eq "gfloat");
    return "Gtk_Clipboard"  if ($c_type eq "Gtk_Clipboard*");
    return "Gtk_Cell_Layout" if ($c_type eq "Gtk_Cell_Layout*");
+   return "Gtk_File_Chooser" if ($c_type eq "Gtk_File_Chooser*");
    return "Gtk_Tree_Path"   if ($c_type eq "Gtk_Tree_Path*");
+   return "Selection_Data"  if ($c_type eq "Gtk_Selection_Data*");
    return "Gtk_Tree_Sortable" if ($c_type eq "Gtk_Tree_Sortable*");
-   return "Gdk_$1"         if ($c_type =~ /Gdk_(.+)\*/);
    return "System.Address" if ($c_type eq "gpointer");
    return "GType"          if ($c_type eq "G_Type");
    return "out Gfloat"     if ($c_type eq "gfloat*");
@@ -522,6 +555,7 @@ sub c_to_low_ada() {
    return "Interfaces.C.Strings.chars_ptr"
                            if ($c_type =~ /g?char\*/ && $param_index == -1);
    return "System.Address" if ($is_object);
+   return "Gdk_$1"         if ($c_type =~ /Gdk_(.+)\*/);
    return &capitalize ($c_type);
 }
 
@@ -535,6 +569,7 @@ sub c_to_call_ada() {
    return "$name & ASCII.NUL"   if ($c_type =~ /g?char\*/);
    return "$name"               if ($c_type eq "GtkTreeIter*");
    return "$name"               if ($c_type eq "GtkTextIter*");
+   return "$name"               if ($c_type eq "GtkSelectionData*");
    if ($is_object) {
       return "Get_Object ($name)";
    }
@@ -566,18 +601,20 @@ sub output_params() {
       print $indent, "  (";
       my ($index) = 0;
       while ($index <= $#args) {
-         my ($n) = &capitalize ($args[$index]);
-         print $indent, "   " if ($index != 0);
-         print $n;
-         print ' ' x ($longuest - length ($n)), " : ";
-         if ($index == 0 && $subprogram_name =~ /^Gtk_New/) {
-            print "out ", &$convert ($arg_types[$index], -1);
-         } elsif ($index == 0 && $subprogram_name =~ /^Initialize/) {
-            print &$convert ($arg_types[$index], $index), "'Class";
-         } else {
-            print &$convert ($arg_types[$index], $index);
+         if ($args[$index] ne "...") {
+            my ($n) = &capitalize ($args[$index]);
+            print $indent, "   " if ($index != 0);
+            print $n;
+            print ' ' x ($longuest - length ($n)), " : ";
+            if ($index == 0 && $subprogram_name =~ /^Gtk_New/) {
+               print "out ", &$convert ($arg_types[$index], -1);
+            } elsif ($index == 0 && $subprogram_name =~ /^Initialize/) {
+               print &$convert ($arg_types[$index], $index), "'Class";
+            } else {
+               print &$convert ($arg_types[$index], $index);
+            }
+            print "", (($index != $#args && $args[$index+1] ne "...") ? ";\n" : ")");
          }
-         print "", (($index != $#args) ? ";\n" : ")");
          $index++;
       }
       print "\n" if ($returns ne "void");
@@ -627,6 +664,7 @@ sub output_subprogram() {
    my ($longuest_param) = 0;
    my ($index);
    my ($is_gobject);
+   my ($has_variable_number_of_args) = 0;
 
    if (!$specs_only && $name eq "Get_Type") {
       ## Nothing to do, this is just a pragma Import
@@ -640,6 +678,14 @@ sub output_subprogram() {
       print "   ", '-' x (length($name) + 6), "\n\n";
    }
 
+   ## Check for multiple number of arguments
+   foreach $a (@args) {
+     if ($a eq "...") {
+       $has_variable_number_of_args = 1;
+       $func = "ada_$func";
+     }
+   }
+
    ## Prototype of Ada subprogram
    print (($returns eq "void") ? "   procedure " : "   function ");
    print $name, "\n";
@@ -773,10 +819,15 @@ sub create_binding() {
    $args =~ s/\s+/ /g;
    $args =~ s/ \* ?/\* /g;
 
-   while ($args =~ / ?(\w+\*?) (\w+) ?[,)]/g) {
-      push (@args, $2);
-      $longuest_param = (length ($2) > $longuest_param) ? length ($2) : $longuest_param;
-      push (@arg_types, $1);
+   while ($args =~ / ?(?:(\w+\*?) (\w+)|(\.\.\.)) ?[,)]/g) {
+      if (defined $3) {
+        push (@args, "...");
+        push (@arg_types, "...");
+      } else {
+        push (@args, $2);
+        $longuest_param = (length ($2) > $longuest_param) ? length ($2) : $longuest_param;
+        push (@arg_types, $1);
+      }
    }
 
    if ($name =~ /^New/) {
