@@ -3,7 +3,7 @@
 --               GtkAda - Ada95 binding for Gtk+/Gnome               --
 --                                                                   --
 --   Copyright (C) 1998-2000 E. Briot, J. Brobecker and A. Charlet   --
---                Copyright (C) 2000-2006 AdaCore                    --
+--                Copyright (C) 2000-2008, AdaCore                   --
 --                                                                   --
 -- This library is free software; you can redistribute it and/or     --
 -- modify it under the terms of the GNU General Public               --
@@ -223,6 +223,15 @@ ada_gtk_list_store_set_int (GtkListStore *list_store,
   gtk_list_store_set (list_store, iter, col, val, -1);
 }
 
+void
+ada_gtk_list_store_set_boolean (GtkListStore *list_store,
+                            GtkTreeIter  *iter,
+                            gint          col,
+                            gboolean          val)
+{
+  gtk_list_store_set (list_store, iter, col, val, -1);
+}
+
 void
 ada_gtk_list_store_set_pixbuf (GtkListStore *list_store,
                             GtkTreeIter  *iter,
@@ -3235,6 +3244,12 @@ ada_gslist_get_data (GSList* list)
   return list->data;
 }
 
+gpointer
+ada_slist_get_data (GSList* list)
+{
+  return list->data;
+}
+
 /******************************************
  ** Functions for Status bar
  ******************************************/
@@ -3793,13 +3808,13 @@ ada_genum_get_value (GEnumValue* value)
   return value->value;
 }
 
-gchar*
+const gchar*
 ada_genum_get_name (GEnumValue* value)
 {
   return value->value_name;
 }
 
-gchar*
+const gchar*
 ada_genum_get_nick (GEnumValue* value)
 {
   return value->value_nick;
@@ -3821,13 +3836,13 @@ ada_gflags_get_value (GFlagsValue* value)
   return value->value;
 }
 
-gchar*
+const gchar*
 ada_gflags_get_name (GFlagsValue* value)
 {
   return value->value_name;
 }
 
-gchar*
+const gchar*
 ada_gflags_get_nick (GFlagsValue* value)
 {
   return value->value_nick;
@@ -3849,6 +3864,12 @@ ada_gparam_get_flags (GParamSpec* param)
   return param->flags;
 }
 
+GType
+ada_gparam_get_owner_type (GParamSpec* param)
+{
+  return param->owner_type;
+}
+
 GType
 ada_gparam_get_value_type (GParamSpec* param)
 {
@@ -4087,9 +4108,10 @@ ada_gparam_ensure_non_null_string (GParamSpecString* param)
  ** New widgets
  ******************************************/
 
-void ada_set_properties_handlers (gpointer klass,
-				  GObjectSetPropertyFunc set_handler,
-				  GObjectGetPropertyFunc get_handler)
+void
+ada_set_properties_handlers (gpointer klass,
+			     GObjectSetPropertyFunc set_handler,
+			     GObjectGetPropertyFunc get_handler)
 {
   G_OBJECT_CLASS (klass)->set_property = set_handler;
   G_OBJECT_CLASS (klass)->get_property = get_handler;
@@ -4164,6 +4186,12 @@ ada_gtype_fundamental (GType type)
   return G_TYPE_FUNDAMENTAL (type);
 }
 
+gboolean
+ada_g_type_is_interface (GType type)
+{
+  return G_TYPE_IS_INTERFACE (type);
+}
+
 /******************************************
  ** Functions for Tree_Iter
  ******************************************/
@@ -4328,3 +4356,161 @@ ada_gtk_text_buffer_insert_with_tags_by_name
   gtk_text_buffer_insert_with_tags_by_name
     (buffer, iter, text, len, name, NULL);
 }
+
+GtkTextTag*
+ada_gtk_text_buffer_create_tag (GtkTextBuffer* buffer, const gchar* name)
+{
+   return gtk_text_buffer_create_tag (buffer, name, NULL);
+}
+
+/***********************************************************
+ ** Gtk_File_Filter
+***********************************************************/
+
+const gchar*
+ada_file_filter_info_get_filename (GtkFileFilterInfo* info)
+{
+  if (info->contains & GTK_FILE_FILTER_FILENAME) {
+    return info->filename;
+  } else {
+    return NULL;
+  }
+}
+
+const gchar*
+ada_file_filter_info_get_uri (GtkFileFilterInfo* info)
+{
+  if (info->contains & GTK_FILE_FILTER_URI) {
+    return info->uri;
+  } else {
+    return NULL;
+  }
+}
+
+const gchar*
+ada_file_filter_info_get_display_name (GtkFileFilterInfo* info)
+{
+  if (info->contains & GTK_FILE_FILTER_DISPLAY_NAME) {
+    return info->display_name;
+  } else {
+    return NULL;
+  }
+}
+
+const gchar*
+ada_file_filter_info_get_mime_type (GtkFileFilterInfo* info)
+{
+  if (info->contains & GTK_FILE_FILTER_MIME_TYPE) {
+    return info->mime_type;
+  } else {
+    return NULL;
+  }
+}
+
+/***********************************************************
+ ** Gtk_File_Chooser_Dialog
+***********************************************************/
+
+GtkWidget *
+ada_gtk_file_chooser_dialog_new
+  (const gchar          *title,
+   GtkWindow            *parent,
+   GtkFileChooserAction  action)
+{
+  return gtk_file_chooser_dialog_new
+    (title, parent, action, NULL, (char *)NULL);
+}
+
+GtkWidget *
+ada_gtk_file_chooser_dialog_new_with_backend
+  (const gchar          *title,
+   GtkWindow            *parent,
+   GtkFileChooserAction  action,
+   const gchar          *backend)
+{
+  return gtk_file_chooser_dialog_new_with_backend
+    (title, parent, action, backend, NULL, (char *)NULL);
+}
+
+/**************************************************************
+ **  Gtk_Message_Dialog
+**************************************************************/
+
+void
+ada_gtk_message_dialog_format_secondary_markup
+  (GtkMessageDialog* dialog, gchar* msg)
+{
+  gtk_message_dialog_format_secondary_markup (dialog, msg);
+}
+
+void
+ada_gtk_message_dialog_format_secondary_text
+  (GtkMessageDialog* dialog, gchar* msg)
+{
+  gtk_message_dialog_format_secondary_text (dialog, msg);
+}
+
+GtkWidget*
+ada_gtk_message_dialog_new
+  (GtkWindow      *parent,
+  GtkDialogFlags  flags,
+  GtkMessageType  type,
+  GtkButtonsType  buttons,
+  const gchar    *message)
+{
+  return gtk_message_dialog_new (parent, flags, type, buttons, message);
+}
+
+GtkWidget*
+ada_gtk_message_dialog_new_with_markup
+  (GtkWindow      *parent,
+  GtkDialogFlags  flags,
+  GtkMessageType  type,
+  GtkButtonsType  buttons,
+  const gchar    *message)
+{
+  return gtk_message_dialog_new_with_markup
+    (parent, flags, type, buttons, message);
+}
+
+/**************************************************************
+ **  Gtk_Bindings
+**************************************************************/
+
+void
+ada_gtk_binding_entry_add_signal_NO
+  (GtkBindingSet* set, guint keyval, GdkModifierType modifier,
+   const gchar* signal_name)
+{
+  gtk_binding_entry_add_signal (set, keyval, modifier, signal_name, 0);
+}
+
+void
+ada_gtk_binding_entry_add_signal_int
+  (GtkBindingSet* set, guint keyval, GdkModifierType modifier,
+   const gchar* signal_name, gint arg1)
+{
+  gtk_binding_entry_add_signal
+    (set, keyval, modifier, signal_name, 1,
+     G_TYPE_INT, arg1);
+}
+
+void
+ada_gtk_binding_entry_add_signal_int_int
+  (GtkBindingSet* set, guint keyval, GdkModifierType modifier,
+   const gchar* signal_name, gint arg1, gint arg2)
+{
+  gtk_binding_entry_add_signal
+    (set, keyval, modifier, signal_name, 2,
+     G_TYPE_INT, arg1, G_TYPE_INT, arg2);
+}
+
+void
+ada_gtk_binding_entry_add_signal_bool
+  (GtkBindingSet* set, guint keyval, GdkModifierType modifier,
+   const gchar* signal_name, gboolean arg1)
+{
+  gtk_binding_entry_add_signal
+    (set, keyval, modifier, signal_name, 1,
+     G_TYPE_BOOLEAN, arg1);
+}
